"""MergeGuide Git hooks module.

Provides local policy enforcement via Git hooks.
"""

from mergeguide.hooks.config import HookConfig, HooksConfig
from mergeguide.hooks.git_utils import (
    get_git_hooks_dir,
    get_repo_root,
    get_staged_files,
    is_git_repo,
)
from mergeguide.hooks.installer import HookExistsError, HookInstaller

__all__ = [
    "HookConfig",
    "HooksConfig",
    "HookExistsError",
    "HookInstaller",
    "get_git_hooks_dir",
    "get_repo_root",
    "get_staged_files",
    "is_git_repo",
]
